/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 *  Jabber
 *  Copyright (C) 1998-1999 The Jabber Team http://jabber.org/
 */

#include "jabber.h"
#include "log.h"

ppdb ppdb_insert(ppdb db, jid id, xmlnode x)
{
    ppdb server, user;

    if(id == NULL || id->server == NULL || x == NULL)
        return db;

    /* this is a server presence packet */
    if(id->user == NULL && id->resource == NULL)
        return _ppdb_insert(db,_ppdb_get(db,id->server),_ppdb_new_node(x, id->server));

    /* set current to the server node */
    server = _ppdb_get(db,id->server);
    if(server == NULL)
        server = db = _ppdb_insert(db,NULL,_ppdb_new_node(NULL,id->server));

    if(id->resource == NULL)
    { /* this is a user presence packet */
        server->branch = _ppdb_insert(server->branch,_ppdb_get(server->branch,id->user),_ppdb_new_node(x,id->user));
        return db;
    }

    /* set current to the user node */
    user = _ppdb_get(server->branch,id->user);
    if(user == NULL)
        user = server->branch = _ppdb_insert(server->branch,NULL,_ppdb_new_node(NULL,id->user));

    /* this is a resource presence packet */
    user->branch = _ppdb_insert(user->branch,_ppdb_get(user->branch,id->resource),_ppdb_new_node(x,id->resource));
    user->primary = _ppdb_primary(user->branch);

    return db;
}

xmlnode ppdb_primary(ppdb db, jid id)
{
    ppdb curr;

    if(db == NULL || id == NULL) return NULL;

    /* travel down the tree as far as the id defines */
    curr = _ppdb_get(db,id->server);
    if(curr != NULL && (id->user != NULL || id->resource != NULL))
        curr = _ppdb_get(curr->branch,id->user);
    if(curr != NULL && id->resource != NULL)
        curr = _ppdb_get(curr->branch,id->resource);

    if(curr == NULL)
        return NULL;

    if(curr->priority >= 0)
        return curr->x;

    if(curr->primary != NULL)
        return curr->primary->x;

    return NULL;
}


xmlnode ppdb_get(ppdb db, jid id)
{
    ppdb curr;
    xmlnode ret;

    if(db == NULL || id == NULL) return NULL;

    if(db->current == NULL)
    {
        /* travel down the tree as far as the id defines, use db->current as the temporary pointer for this get loop */
        db->current = _ppdb_get(db,id->server);
        if(db->current != NULL && id->user != NULL || id->resource != NULL)
            db->current = _ppdb_get(db->current->branch,id->user);
        if(db->current != NULL && id->resource != NULL)
            db->current = _ppdb_get(db->current->branch,id->resource);

        if(db->current == NULL) return NULL;

        if(db->current != db)
            db->current->current = db->current->branch;
        if(db->current->x != NULL) return db->current->x; /* send the node's presence as well as it's children's */
    }
    curr = db->current;

    if(curr == db) return NULL;

    for(; curr->current != NULL && curr->current->x == NULL; curr->current = curr->current->next);

    if(curr->current == NULL)
    {
        db->current = NULL;
        return NULL;
    }

    ret = curr->current->x;
    curr->current = curr->current->next;

    return ret;
}


void ppdb_free(ppdb db)
{
    ppdb curr, next;

    if(db == NULL) return;

    curr = db;
    while(curr != NULL)
    {
        next = curr->next;
        ppdb_free(curr->branch);
        pool_free(curr->p);
        curr = next;
    }
}

ppdb _ppdb_new(pool p)
{
    ppdb newdb;

    if(p == NULL) p = pool_new();

    newdb = pmalloc(p,sizeof(_ppdb));
    memset(newdb,'\0',sizeof(_ppdb));
    newdb->p = p;
    newdb->priority = -1;

    return newdb;
}

ppdb _ppdb_new_node(xmlnode x, char *node)
{
    char *pri;
    ppdb newdb;

    x = xmlnode_dup(x);

    newdb = _ppdb_new(xmlnode_pool(x));
    newdb->node = pstrdup(newdb->p, node);
    newdb->x = x;

    if(x != NULL && xmlnode_get_attrib(x,"type") == NULL)
    {
        newdb->priority = 0;
        pri = xmlnode_get_data(xmlnode_get_firstchild(xmlnode_get_tag(x,"priority")));
        if(pri != NULL && atoi(pri) > 0) newdb->priority = atoi(pri);
    }

    return newdb;
}

ppdb _ppdb_get(ppdb list, char *node)
{
    ppdb step;

    if(list == NULL) return NULL;

    for(step = list; step != NULL; step = step->next)
    {
        if(node == NULL && step->node == NULL) return step;
        if(j_strcmp(node,step->node) == 0) return step;
    }

    return NULL;
}

ppdb _ppdb_primary(ppdb list)
{
    ppdb ret = NULL, curr;
    int currp = -1;

    for(curr = list; curr != NULL; curr = curr->next)
        if(curr->priority > currp)
        {
            ret = curr;
            currp = ret->priority;
        }

    return ret;
}

ppdb _ppdb_insert(ppdb list, ppdb old, ppdb new)
{
    ppdb branch;

    if(list == NULL)
        return new;

    if(old != NULL)
    {
        new->branch = old->branch;
        new->primary = old->primary;
        list = _ppdb_free(list, old);
    }

    new->next = list;
    return new;
}

ppdb _ppdb_free(ppdb db, ppdb old)
{
    ppdb ret = db, curr;

    if(db == old)
        ret = old->next;

    for(curr = db; curr->next != NULL && curr->next != old; curr = curr->next);
    if(curr->next != NULL) curr->next = old->next;

    pool_free(old->p);

    return ret;
}

